/*
 * @file http_client.h
 * @author Chris GauthierDickey
 * @brief Defines the functions that are required for the project.
 *
 * This file defines all the functions required for the project. You
 * will need to fill in the bodies in the appropriate C functions and
 * complete the project according to the directions.
 *  
 */
#ifndef __HTTP_CLIENT_H__
#define __HTTP_CLIENT_H__


/* lots of includes to try to make sure we cover the bases */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/select.h>
#include <poll.h>
#include <sys/event.h>
#include <strings.h>
#include <pthread.h>

#define MHC_YES 1
#define MHC_NO 0

struct MHC_client;
 

/**
 * Reads in n bytes from the descriptor. Note that with a socket,
 * the read may not actually read in all the bytes. However, this
 * function ensures that when we specify nbytes, that many bytes
 * are actually returned.
 *
 * @param fd  The file descriptor being read
 * @param buff  A pointer to the buffer that will be filled during
 *              the read. We assume that it is of sufficient size.
 * @param nbytes  The number of bytes to read before returning.
 * @return The number of bytes read (should be nbytes, or -1
 *         if an error occurred.
 */
ssize_t readn(int fd, char *buff, size_t nbytes);





/**
 * Writes n bytes to the descriptor. Note that with a socket,
 * the write may not actually write all the bytes that you give it.
 * However, this function ensures that when we specify nbytes, that 
 * many bytes are actually written.
 *
 * @param fd  The file descriptor being written to
 * @param buff  A pointer to the buffer that will be read from during
 *              the write.
 * @param nbytes  The number of bytes to write before returning.
 * @return The number of bytes written (should be nbytes, unless an
 *         error occurred, 0 if the end-of-file was reached, or -1
 *         if an error occurred.
 */
ssize_t writen(int fd, const char *buff, size_t nbytes);





/**
 * Creates an MHC_client and connects it to the site on the given port.
 * The socket is created at this point once the site and port are valid.
 * Most likely you will use getaddrinfo before you create the socket so
 * that you can easily specify what address is needed. 
 *
 * @param client A pointer to an allocated (stack or heap) MHC_client which
 *               MHC_connect will fill as it connects. 
 * @param site The string name of the site to connect to, such as google.com
 * @param port The port number to connect to, like 80, in string format
 */
void MHC_connect(MHC_client *mhc, const char *site, const char *port)





/**
 * Closes the MHC_client, terminating the connection.
 * 
 * @param mhc A pointer to the MHC_client which is being closed
 * @return MHC_YES if it was successful, or MHC_NO, if it was not
 */
int MHC_close(MHC_client *mhc);





/**
 * Sends an HTTP 1.1 GET message over the connection.
 *
 * @param mhc The MHC_client we're executing the get on
 * @param name The name of the file, including path, we're getting
 * @return MHC_YES if it was successful, or MHC_NO if it wasn't
 */
void MHC_get(MHC_client *mhc, const char *name); 




/**
* Reads from the socket held by MHC_client into the buffer. select()
* may be used to prevent the call from blocking (that, or pthreads)
* since no data may be ready on the socket for reading.
*
* @param mhc The MHC_client we're reading from
* @param buf A character buffer
* @param len Maximum number of bytes to read into buf
*/
void MHC_read(MHC_client *mhc, char *buf, size_t len);

#endif 
